@@ -8,6 +8,7 @@
 #include <libxml++/nodes/node.h>
 #include <libxml++/exceptions/internal_error.h>
 #include <libxml/xpath.h>
+#include <libxml/xpathInternals.h>
 #include <libxml/tree.h>
 
 #include <iostream>
@@ -24,7 +25,7 @@ Node::Node(xmlNode* node)
 Node::~Node()
 {}
 
-Node::NodeList Node::get_children(const std::string& name)
+Node::NodeList Node::get_children(const Glib::ustring& name)
 {
    xmlNode* child = impl_->children;
    if(!child)
@@ -54,13 +55,13 @@ Node::NodeList Node::get_children(const std::string& name)
    return children;
 }
 
-const Node::NodeList Node::get_children(const std::string& name) const
+const Node::NodeList Node::get_children(const Glib::ustring& name) const
 {
   return const_cast<Node*>(this)->get_children(name);
 }
 
-Element* Node::add_child(const std::string& name,
-                         const std::string& ns_prefix)
+Element* Node::add_child(const Glib::ustring& name,
+                         const Glib::ustring& ns_prefix)
 {
    xmlNode* node = 0;
    xmlNs* ns = 0;
@@ -111,12 +112,12 @@ Node* Node::import_node(const Node* node, bool recursive)
   return static_cast<Node*>(imported_node->_private);
 }
 
-std::string Node::get_name() const
+Glib::ustring Node::get_name() const
 {
   return impl_->name ? (const char*)impl_->name : "";
 }
 
-void Node::set_name(const std::string& name)
+void Node::set_name(const Glib::ustring& name)
 {
   xmlNodeSetName( impl_, (const xmlChar *)name.c_str() );
 }
@@ -137,62 +138,115 @@ const xmlNode* Node::cobj() const
   return impl_;
 }
 
-std::string Node::get_path() const
+Glib::ustring Node::get_path() const
 {
   xmlChar* path = xmlGetNodePath(impl_);
-  std::string retn = path ? (char*)path : "";
+  Glib::ustring retn = path ? (char*)path : "";
   xmlFree(path);
   return retn;
 }
 
-NodeSet Node::find(const std::string& xpath) const
+static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
 {
-  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
-  ctxt->node = impl_;
   xmlXPathObject* result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
 
-  if (result->type != XPATH_NODESET)
+  if(!result)
+  {
+    xmlXPathFreeContext(ctxt);
+    throw exception("Invalid XPath: " + xpath);
+  }
+
+  if(result->type != XPATH_NODESET)
   {
     xmlXPathFreeObject(result);
     xmlXPathFreeContext(ctxt);
-    throw internal_error("sorry, only nodeset result types supported for now.");
+    throw internal_error("Only nodeset result types are supported.");
   }
 
   xmlNodeSet* nodeset = result->nodesetval;
   NodeSet nodes;
-  if( nodeset ) {
+  if( nodeset )
+  {
+    nodes.reserve( nodeset->nodeNr );
     for (int i = 0; i != nodeset->nodeNr; ++i)
       nodes.push_back(static_cast<Node*>(nodeset->nodeTab[i]->_private));
   }
-  else {
+  else
+  {
     // return empty set
   }
+
   xmlXPathFreeObject(result);
   xmlXPathFreeContext(ctxt);
 
   return nodes;
 }
 
-std::string Node::get_namespace_prefix() const
+NodeSet Node::find(const Glib::ustring& xpath) const
 {
+  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
+  ctxt->node = impl_;
+  
+  return find_impl(ctxt, xpath);
+}
+
+NodeSet Node::find(const Glib::ustring& xpath,
+		   const PrefixNsMap& namespaces) const
+{
+  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
+  ctxt->node = impl_;
+
+  for (PrefixNsMap::const_iterator it=namespaces.begin();
+       it != namespaces.end(); it++)
+    xmlXPathRegisterNs(ctxt,
+		       reinterpret_cast<const xmlChar*>(it->first.c_str()),
+		       reinterpret_cast<const xmlChar*>(it->second.c_str()));
+
+  return find_impl(ctxt, xpath);
+}
+
+Glib::ustring Node::get_namespace_prefix() const
+{
+  if(impl_->type == XML_DOCUMENT_NODE)
+  {
+    //impl_ is actually of type xmlDoc, instead of just xmlNode.
+    //libxml does not always use GObject-style inheritance, so xmlDoc does not have all the same struct fields as xmlNode.
+    //Therefore, a call to impl_->ns would be invalid.
+    //This can be an issue when calling this method on a Node returned by Node::find().
+    //See the TODO comment on Document, suggesting that Document should derived from Node.
+
+    return Glib::ustring();
+  }
+
   if(impl_ && impl_->ns && impl_->ns->prefix)
     return (char*)impl_->ns->prefix;
   else
-    return std::string();
+    return Glib::ustring();
 }
 
-std::string Node::get_namespace_uri() const
+Glib::ustring Node::get_namespace_uri() const
 {
+  if(impl_->type == XML_DOCUMENT_NODE)
+  {
+    //impl_ is actually of type xmlDoc, instead of just xmlNode.
+    //libxml does not always use GObject-style inheritance, so xmlDoc does not have all the same struct fields as xmlNode.
+    //Therefore, a call to impl_->ns would be invalid.
+    //This can be an issue when calling this method on a Node returned by Node::find().
+    //See the TODO comment on Document, suggesting that Document should derived from Node.
+
+    return Glib::ustring();
+  }
+
    if(impl_ && impl_->ns && impl_->ns->href)
     return (char*)impl_->ns->href;
   else
-    return std::string();
+    return Glib::ustring();
 }
 
-void Node::set_namespace(const std::string& ns_prefix)
+void Node::set_namespace(const Glib::ustring& ns_prefix)
 {
   //Look for the existing namespace to use:
-  xmlNs* ns = xmlSearchNs( cobj()->doc, cobj(), (xmlChar*)ns_prefix.c_str() );
+  xmlNs* ns = xmlSearchNs( cobj()->doc, cobj(), (xmlChar*)(ns_prefix.empty() ? 0 : ns_prefix.c_str()) );
   if(ns)
   {
       //Use it for this element:
