@@ -76,6 +76,11 @@ void Parser::release_underlying()
   {
     context_->_private = 0; //Not really necessary.
     
+    if( context_->myDoc != NULL )
+    {
+      xmlFreeDoc(context_->myDoc);
+    }
+
     xmlFreeParserCtxt(context_);
     context_ = 0;
   }
@@ -189,7 +194,7 @@ void Parser::check_for_exception()
   
   if(exception_)
   {
-	  std::auto_ptr<exception> tmp ( exception_ );
+    std::auto_ptr<exception> tmp ( exception_ );
     exception_ = 0;
     tmp->Raise();
   }
